import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class ControlCanvas extends ScaleCanvas implements MouseListener,KeyListener {
    Manager M; //communicates with the rest of the program
    ControlPanel MAIN,PROOFMODE,DEBUG,INIT,ELIM,DISPLAY; //control panels
    Parameter[][] ENERGY=new Parameter[3][2]; //controls which energy hybrid is tested
    SelectInteger ENERGY1,ENERGY2;
    Proof PROOF;    //the object that runs the proof
    ProofMonitor PC; //displays proof status
    ListenSquare GO,HALT;  //stop/go   
    PopupManager PM;  //controls the popup windows
    ListenSquare[] INFO=new ListenSquare[4];  //question (info) boxes
    Lever SPEED,PARALLEL,PARALLEL2;  //controls the speed of the proof calculation

    /*This main routine makes the window*/

     public ControlCanvas() {
	 addMouseListener(this);
	 addKeyListener(this);
	 ControlPanelSetup.main(this);
         setButtons();
	 PM=new PopupManager(null,110,0);
         PROOF=new Proof();   
	 PC=new ProofMonitor(200,0);
	 SPEED=new Lever(200,205,0,20);
	 PARALLEL=new Lever(200,235,0,8);
	 PARALLEL2=new Lever(370,235,1,2);
	 for(int i=0;i<8;++i) PARALLEL.on[i]=1;

	 String[][] c1={{"1","4"},{"0","0"},{"0","0"}};

	 for(int i=0;i<3;++i) {
	    ENERGY[i][0]=new Parameter(293+90*i,303,52,c1[i][0]);
	    ENERGY[i][1]=new Parameter(340+90*i,303,25,c1[i][1]);
	 }
	 ENERGY2=new SelectInteger(515,333,40,20,0,0,6,1);
	 INFO[0]=new ListenSquare(178,0,12,12); //question box
	 INFO[1]=new ListenSquare(582,0,12,12); //question box
	 INFO[2]=new ListenSquare(335,237,12,12); //question box
	 INFO[3]=new ListenSquare(528,207,12,12); //question box
     }

    public void setButtons() {
	GO=new ListenSquare(150,320,25,20,Color.red); GO.on=1;
	HALT=new ListenSquare(175,320,35,20,Color.red); HALT.on=1;	
    }


   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      makeFrame(g);
      g.setFont(new Font("Helvetica",Font.PLAIN,12));
      drawPanels(g);    
      drawButtons(g);
      PM.render(g); 
      INFO[0].infoRender(g);
      INFO[1].infoRender(g);   
      INFO[2].infoRender(g);   	   
      INFO[3].infoRender(g);   	  
   }

    public void makeFrame(Graphics2D g) {
    g.setColor(new Color(100,0,160));
      g.fillRect(0,0,getWidth(),getHeight()); 
    

    }

    public void drawButtons(Graphics2D g) {
      g.setFont(new Font("Helvetica",Font.PLAIN,12));
      if(PROOF.HALT==true) { 
          GO.render(g,new Color(0,180,0));
          g.drawString("go",(int)(GO.x+5),(int)(GO.y+15));
      }
      if(PROOF.HALT==false) { 
          HALT.render(g,new Color(220,0,0));
          g.drawString("stop",(int)(HALT.x+5),(int)(HALT.y+15));
      }
    }

    /**This draws the control panels*/
    public void drawPanels(Graphics2D g) {
       DISPLAY.render(g,0,210,100);
       MAIN.render(g,0,0,100);
       if(MAIN.mode==0) {
            PROOFMODE.render(g,0,55,100);
            ELIM.render2(g,0,110,100);
	    PC.render(g);
       }
       if(MAIN.mode==1) DEBUG.render(g,200,0,160);
       ENERGY2.render(g,Color.blue,Color.white,new Color(0,0,0,0));
       g.setColor(Color.white);	
       g.setFont(new Font("Helvetica",Font.PLAIN,16));
       g.drawString("controls which energy hybrid we test",295,297);
       g.drawString("use arrows for energy presets ",295,347);
       for(int i=0;i<3;++i) {
	   for(int j=0;j<2;++j) {
	       ENERGY[i][j].render(g);
	   }
       }
       g.setFont(new Font("Helvetica",Font.PLAIN,15));
       SPEED.render(g,"",new Color(0,200,0));
       PARALLEL.render2(g,"",new Color(155,0,155),new Color(255,180,0));
       PARALLEL2.render(g,"",new Color(155,0,155),new Color(255,180,0));
       g.drawString("speed",(int)(SPEED.x-50),(int)(SPEED.y+12));
       g.drawString("parallelize",(int)(PARALLEL.x-74),(int)(PARALLEL.y+12));
       g.drawString("all on/off",(int)(PARALLEL2.x+40),(int)(PARALLEL2.y+12));
  }

  public void updatePanels(Point X) { 
      if(PROOF.HALT==false) return;  
      if(MAIN.mode==0) PROOFMODE.switchMode(X);
      if(MAIN.mode==1) DEBUG.switchMode(X); 
      MAIN.switchMode(X);
  }


 
    /**Gets the energy combo to be tested*/
   public int[][] getEnergy() {
    int[][] E=new int[3][2];
    for(int i=0;i<3;++i) {
        for(int j=0;j<2;++j) {
        E[i][j]=ENERGY[i][j].getVal();
        }
    }
    return E;
   }

    /**gets the type of the energy*/
    public int getType() {
	int[][] E=getEnergy();
	return ProofGrading.getType(E);
    }
 
    /**Tells which elimination features to use in the test*/
 
      public int[] eliminationSpecs() {
    int[] elim=new int[5];
    for(int i=0;i<5;++i) elim[i]=M.C.ELIM.L[i].on;
        return elim;
    }

     public void doRun() {
	 if(MAIN.mode==0) doRunProof();
	 if(MAIN.mode==1) doRunDebug();
     }


     public void doRunProof() {
	 if(PROOF.HALT==false) return;
         PROOF=new Proof(this.M);
	 new Thread(PROOF).start();
    }

     public void doRunDebug() {
	 Debug.run(DEBUG.mode,M);
    }


    public void doStop() {
	PROOF.HALT=true;
    }

  public void documentStuff(Point X) {
        if(PROOFMODE.L[20].inside(X)==1) DocumentControl.proofmode(M.D);
        if(MAIN.L[20].inside(X)==1) DocumentControl.action(M.D);
        if(ELIM.L[20].inside(X)==1) DocumentControl.grade(M.D);
	if(DEBUG.L[20].inside(X)==1) DocumentControl.debugmode(M.D);
	if(INFO[0].inside(X)==1) DocumentControl.popups(M.D);
	if(INFO[1].inside(X)==1) DocumentControl.monitor(M.D);
	if(INFO[2].inside(X)==1) DocumentControl.parallel(M.D);
	if(INFO[3].inside(X)==1) DocumentControl.speed(M.D);
    }

    public void setEnergy(MouseEvent e) {
      for(int i=0;i<3;++i) {
	 for(int j=0;j<2;++j) {
	    ENERGY[i][j].process(e);
	 }
      }   
      MouseData J=MouseData.process(e);
      ENERGY2.modify(J.X);
      if(ENERGY2.isModified(J.X)==1) {
	  Integer t=new Integer(ENERGY2.val);
          if(t==0) EnergyPresets.G(4,ENERGY);
          if(t==1) EnergyPresets.G5Flat(ENERGY);
          if(t==2) EnergyPresets.G(6,ENERGY);
	  if(t==3) EnergyPresets.G10SharpSharp(ENERGY);
	  if(t==4) EnergyPresets.G10Sharp(ENERGY);

	  /**sets the test options correctly*/
	  M.C.ELIM.L[0].on=1;
	  M.C.ELIM.L[1].on=1;
	  
	  if(t<3) {
	      M.C.ELIM.L[2].on=0;
	      M.C.ELIM.L[3].on=0;
	  }

	  if(t==3) {
	      M.C.ELIM.L[2].on=0;
	      M.C.ELIM.L[3].on=1;
	  }

	  if(t==4) {
	      M.C.ELIM.L[2].on=1;
	      M.C.ELIM.L[3].on=0;
	  }
      }
    }

    public void setParallel(Point X) {
	PARALLEL.toggle(X);
	int t=PARALLEL2.getValue(X);
	if(t!=-1) {
	    for(int i=0;i<8;++i) PARALLEL.on[i]=t;
	}
    }
    
    public void mouseClicked(MouseEvent e) { 
	PM.M=this.M;
        MouseData J=MouseData.process(e);
	PM.process(J.X);
        updatePanels(J.X); 
	documentStuff(J.X);
        if(GO.inside(J.X)==1) doRun();
        if(HALT.inside(J.X)==1) doStop();
	setEnergy(e);
	SPEED.process(J.X);
	setParallel(J.X);
	ELIM.toggle(J.X);
	int test=DISPLAY.toggle(J.X);
	if(test==20) DocumentBlock.connect(M.D);
        M.repaint();
    }

    public void doTest() {
	Box B=new Box(new Complex(0,0),0,0);
	for(int i=0;i<4;++i) {
	    Box C=B.subdivide0(i);
	    C.g.print();
	}
    }
    



    public void mousePressed(MouseEvent e) {}
     public void mouseReleased(MouseEvent e) {}
     public void mouseEntered(MouseEvent e) {
	 requestFocus();
     }
     public void mouseExited(MouseEvent e) {}   

    public void keyTyped(KeyEvent e) {
	for(int i=0;i<3;++i) {
	   ENERGY[i][0].processKey(false,e);
	   ENERGY[i][1].processKey(true,e);
	}
	M.repaint();
    }
 
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}



}

