import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**   The HALT variable says whether the program is running.
   LIST is the list of configurations
   COUNT is the length of the list
   PASSED keeps track of the number passed;
   TOTAL keeps track of the volume of blocks passed.*/
   

public class Proof implements Runnable {
    ControlCanvas C;
    boolean HALT;
    Block[] LIST=new Block[300];
    int COUNT;
    int MAXCOUNT;
    long TIME;
    long PASSED;
    double TOTAL;
    int MAXDEPTH;
    int[] LOG=new int[300];
    int[] CURRENTDEPTH=new int[4];

    public Proof() {
	HALT=true;
    }

    /**This is the construction method for the program*/

    public Proof(ControlCanvas CC) {
	this.C=CC;
	HALT=true;
	COUNT=1;
	TOTAL=0;
	PASSED=0;
        LIST[0]=Block.initialBox();
	for(int i=0;i<200;++i) LOG[i]=0;		
    }


    /**Here is the runnable routine*/

    public void run() {	
        HALT=false;
	TIME=System.currentTimeMillis();
	sendMessage();
  	int[][] E=C.getEnergy();
	int[] elim=C.getEliminationSpecs();

 	while((HALT==false)&&(COUNT>0)) {
	    mainStep(elim,E);
	    sendMessage();
	}
	HALT=true;
    }


    /**The main step in the algorithm.*/

    public void mainStep(int[] elim,int[][] E) {   
 	 int[] test={0,0};
         Block X=new Block(LIST[COUNT-1]);
	 test=ProofGrading.main(elim,E,X);
	 if(test[0]==0)  subdivide(X,test[1]);
	 if(test[0]==1)  eliminate(X);  
	 if(MAXCOUNT<COUNT) MAXCOUNT=COUNT;
         displayTime();
    }

    /** elimination: we display the eliminated block
        on the picture window*/

    public void eliminate(Block X) {
	--COUNT;
	++LOG[14+X.volumeLog()];  //14 because [0,4] x [-2,2]^2 x [-2,2]^2 x [-2,2]^2 has vol 2^14
        for(int i=0;i<4;++i) CURRENTDEPTH[i]=X.B[i].k;
	int depth=X.maxDepth();
	if(MAXDEPTH<depth) MAXDEPTH=depth;
	++PASSED;
    }

    /**Subdivision: The integer q tells which subdivision to perform.**/

    public void  subdivide(Block X,int q) {
	--COUNT;
	if(q==0) {
	   for(int n=0;n<2;++n) {
	      LIST[COUNT]=X.subdivide1(n);
	      ++COUNT;
	   }
	}

	if(q!=0) {
	  for(int n=0;n<4;++n) {
	  LIST[COUNT]=X.subdivide0(q,n);
	  ++COUNT;
	  }
	}
    }

    /**These routines aren't part of the proof, but they are used to 
       mark the progress of the program.**/

    public void displayTime() {
	 double[] time=new double[10];
         double tt=System.currentTimeMillis()-TIME;
	 sendMessage();
    }


    /**The message sent to the main window*/

    public void sendMessage() {
	computeVolume();
	String T=Time.convert(System.currentTimeMillis()-TIME);
	C.S[0]="time "+T;
	Long c=new Long(COUNT);
	C.S[1]="current list depth: "+c.toString();
	Double v=new Double(TOTAL);
	C.S[2]="volume "+v.toString();
	C.repaint();
	c=new Long(PASSED);
	C.S[3]="partition size "+c.toString();
	C.repaint();
    }

    public void computeVolume() {
         double total=0;
	 for(int i=0;i<200;++i) {
	   total=total+LOG[i]*Math.pow(.5,i);
	 } 
         TOTAL=total;
    }

}


